package calcolo_media;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Oggetto che recupera i dati prodotti dal Produttore.
 *
 * @author <a href="mailto:ervincio@tiscali.it">Vincenzo Manzoni</a>
 */

public class Consumatore extends Thread {

        /** Variabile per la condivisione dei dati Produttori - Consumatori. */
        private Buffer data;

        /** Lista che mi conterr i dati prelevati dal buffer. */
        private ArrayList list = new ArrayList();

        ThreadGroup produttori;

        /**
         * Istanzia un Consumatore, attribuendogli un nome, una variabile
         * per lo scambio di dati con i produttori e il riferimento al gruppo
         * di thread produttori.
         *
         * @param name Nome identificativo del Consumatore.
         *             Dovrebbe essere nelle forma CONSUMATOREn, dove
         *             n rappresenta un numero naturale.
         * @param data Variabile, di tipo Buffer, per lo scambio di dati con i
         *             produttori.
         * @param produttori Riferimento al thread dei produttori
         */
        public Consumatore(String name, Buffer data, ThreadGroup produttori) {
                super(name);
                this.data = data;
                this.produttori = produttori;
        }

        /**
         * Recupera dal buffer gli oggetti di tipo Integer che i produttori hanno
         * piazzato dentro.
         * Prima di recuperarli verifica che la variabile di scambio non sia vuota.
         * Se lo  si mette in <code>wait()</code>.
         */
        public void run() {

                // Variabile in cui si mettere gli oggetti di tipo Integer recuperati
                Integer i = null;
                boolean running = true;

                // Ciclo infinito
                while(running) {
                        try {

                                // Lock sulla variabile
                                synchronized(data) {

                                        // Se il buffer  vuoto vado in wait()
                                        if(data.isEmpty())
                                                data.wait();
                                        // Altrimenti recupero l'oggetto e lo metto nella lista
                                        else {
                                                i = (Integer) data.remove();
                                                list.add(i);

                                                /* Se raggiungo la dimensione massima (10), interrompo
                                                 * l'esecuzione del thread
                                                 */
                                                if(list.size() == 10) {
                                                        produttori.interrupt();
                                                        running = false;
                                                }
                                        }
                                }

                                /*
                                 * N.B.: In questo caso non uso lo Sleep, perch se il dato
                                 * non  pronto vado in wait() e mi risveglio non appena il
                                 * produttore mi manda un messaggio di notify()
                                 */
                        }
                        catch (InterruptedException ie) {
                        }
                }
        }

        /**
     * Calcola la media dei numeri prelevati dal buffer
     *
     * @return La media dei numeri generati dai produttori
     */
        public double getMedia() {
                Iterator i = list.iterator();
                int cicli = 0;

                double media = 0;

                while(i.hasNext()) {
                        cicli ++;
                        media += ((Integer)(i.next())).doubleValue();
                }

                return media/cicli;
        }
}
